# pyhamcrest-qt - PyHamcrest extensions for use with Qt (either pyside2 or PyQt5)
# Copyright (C) 2020 Kamil 'konserw' Strzempowicz, konserw@gmail.com
# This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of the License, or any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this program.
# If not, see <http://www.gnu.org/licenses/>.
#
from hamcrest.core.base_matcher import BaseMatcher
from hamcrest.core.description import Description
from Qt5 import QtCore


class HasFlags(BaseMatcher):
    def __init__(self, names, expected):
        self.names = names
        self.expected = expected

    def _matches(self, item) -> bool:
        return item == self.expected

    def describe_to(self, description: Description) -> None:
        description.append_text(f"flags: {self._qt_flags_to_str(self.expected)}")

    def describe_mismatch(self, item, mismatch_description: Description) -> None:
        mismatch_description.append_text(f"was {self._qt_flags_to_str(item)}")

    def _qt_flags_to_str(self, flags):
        str_list = []
        for flag, name in self.names.items():
            if flag & flags:
                str_list.append(name)
        return "( {} )".format(" | ".join(str_list))


def has_item_flags(flags: QtCore.Qt.ItemFlags) -> HasFlags:
    names = {
        QtCore.Qt.NoItemFlags: "NoItemFlags",
        QtCore.Qt.ItemIsSelectable: "ItemIsSelectable",
        QtCore.Qt.ItemIsEditable: "ItemIsEditable",
        QtCore.Qt.ItemIsDragEnabled: "ItemIsDragEnabled",
        QtCore.Qt.ItemIsDropEnabled: "ItemIsDropEnabled",
        QtCore.Qt.ItemIsUserCheckable: "ItemIsUserCheckable",
        QtCore.Qt.ItemIsEnabled: "ItemIsEnabled",
        QtCore.Qt.ItemIsAutoTristate: "ItemIsAutoTristate",
        QtCore.Qt.ItemNeverHasChildren: "NeverHasChildren",
        QtCore.Qt.ItemIsUserTristate: "ItemIsUSerTristate",
    }
    return HasFlags(names, flags)


def has_window_type(window_type: QtCore.Qt.WindowType) -> HasFlags:
    names = {
        QtCore.Qt.Widget: "Widget",
        QtCore.Qt.Window: "Window",
        QtCore.Qt.Dialog: "Dialog",
        QtCore.Qt.Sheet: "Sheet",
        QtCore.Qt.Drawer: "Drawer",
        QtCore.Qt.Popup: "Popup",
        QtCore.Qt.Tool: "Tool",
        QtCore.Qt.ToolTip: "ToolTip",
        QtCore.Qt.SplashScreen: "Splashscreen",
        QtCore.Qt.Desktop: "Desktop",
        QtCore.Qt.SubWindow: "SubWindow",
        QtCore.Qt.ForeignWindow: "ForeignWindow",
        QtCore.Qt.CoverWindow: "CoverWindow"
    }
    return HasFlags(names, window_type)



# pyhamcrest-qt - PyHamcrest extensions for use with Qt (either pyside2 or PyQt5)
# Copyright (C) 2020 Kamil 'konserw' Strzempowicz, konserw@gmail.com
# This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of the License, or any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this program.
# If not, see <http://www.gnu.org/licenses/>.
#
from distutils.core import setup

setup(
    name="pyhamcrest-qt",
    packages=["qtmatchers"],
    version="0.3",
    license="GNU GPL v3+",
    description="PyHamcrest extensions for use with Qt (either pyside2 or PyQt5)",
    author="Kamil Strzempowicz",
    author_email="konserw@gmail.com",
    url="https://github.com/konserw/pyhamcrest-qt",
    download_url="https://github.com/konserw/pyhamcrest-qt/archive/v0.3.zip",
    keywords=["Hamcrest", "pyHamcrest", "Qt", "pyside2", "PyQt5", "matchers"],
    install_requires=[
        "Qt5.py",
        "PyHamcrest",
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",  # "3 - Alpha", "4 - Beta" or "5 - Production/Stable"
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Testing",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Programming Language :: Python :: 3",
    ],
)



# pyhamcrest-qt - PyHamcrest extensions for use with Qt (either pyside2 or PyQt5)
# Copyright (C) 2020 Kamil 'konserw' Strzempowicz, konserw@gmail.com
# This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of the License, or any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this program.
# If not, see <http://www.gnu.org/licenses/>.
#
from hamcrest.core.base_matcher import BaseMatcher
from hamcrest.core.description import Description


class HasProperty(BaseMatcher):
    def __init__(self, function, expected, desc_true, desc_false):
        self.function = function
        self.expected = expected
        self.desc = {
            True: desc_true,
            False: desc_false
        }

    def _matches(self, item) -> bool:
        return getattr(item, self.function)() == self.expected

    def describe_to(self, description: Description) -> None:
        description.append_text(f"widget to be {self.desc[self.expected]}.")

    def describe_mismatch(self, item, mismatch_description: Description) -> None:
        mismatch_description.append_text(f"was {self.desc[not self.expected]}.")


def enabled():
    return HasProperty("isEnabled", True, "enabled", "disabled")


def disabled():
    return HasProperty("isEnabled", False, "enabled", "disabled")
